@@ -11,29 +11,90 @@ from autotrader.brokers.oanda import utils
 import datetime
 import pandas as pd
 import numpy as np
-
+import traceback
+import sys
+import time
 
 class Oanda():
     def __init__(self, oanda_config, utils):
         ''' Create v20 context. '''
-        API             = oanda_config["API"]
-        ACCESS_TOKEN    = oanda_config["ACCESS_TOKEN"]
-        port            = oanda_config["PORT"]
+        self.API             = oanda_config["API"]
+        self.ACCESS_TOKEN    = oanda_config["ACCESS_TOKEN"]
+        self.port            = oanda_config["PORT"]
         self.ACCOUNT_ID = oanda_config["ACCOUNT_ID"]
-        self.api        = v20.Context(hostname=API, 
-                                      token=ACCESS_TOKEN, 
-                                      port=port)
+        self.api        = v20.Context(hostname = self.API, 
+                                      token = self.ACCESS_TOKEN, 
+                                      port = self.port)
         
-        STREAM_API      = "stream-fxpractice.oanda.com"
-        self.stream     = v20.Context(hostname=STREAM_API, 
-                                      token=ACCESS_TOKEN, 
-                                      port=443)
+        self.STREAM_API = "stream-fxpractice.oanda.com"
+        self.stream     = v20.Context(hostname = self.STREAM_API, 
+                                      token = self.ACCESS_TOKEN, 
+                                      port = self.port)
         
         self.open_positions     = {}
         
     
+    def check_connection(self):
+        '''
+        Connects to Oanda v20 REST API. An initial call is performed to check
+        for a timeout error.
+        '''
+        
+        for atempt in range(10):
+            try:
+                # Attempt basic task to check connection
+                self.api.account.get(accountID=self.ACCOUNT_ID)
+            
+            except BaseException as ex:
+                # Error has occurred
+                ex_type, ex_value, ex_traceback = sys.exc_info()
+            
+                # Extract unformatter stack traces as tuples
+                trace_back = traceback.extract_tb(ex_traceback)
+            
+                # Format stacktrace
+                stack_trace = list()
+            
+                for trace in trace_back:
+                    trade_string = "File : %s , Line : %d, " % (trace[0], trace[1]) + \
+                                   "Func.Name : %s, Message : %s" % (trace[2], trace[3])
+                    stack_trace.append(trade_string)
+                
+                print("\nWARNING FROM OANDA API: The following exception was caught.")
+                print("Time: {}".format(datetime.datetime.now().strftime("%b %d %H:%M:%S")))
+                print("Exception type : %s " % ex_type.__name__)
+                print("Exception message : %s" %ex_value)
+                print("Stack trace : %s" %stack_trace)
+                print("  Attempting to reconnect to Oanda v20 API.")
+                
+                time.sleep(3)
+                api = v20.Context(hostname = self.API, 
+                                  token = self.ACCESS_TOKEN, 
+                                  port = self.port)
+                self.api = api
+            
+            else:
+                break
+            
+        else:
+            print("FATAL: All attempts to connect to Oanda API have failed.")
+        
+        
+    def get_NAV(self):
+        ''' Returns Net Asset Value of account. '''
+        
+        self.check_connection()
+        
+        response = self.api.account.get(accountID=self.ACCOUNT_ID)
+        
+        return response.body["account"].NAV
+    
+    
     def get_price(self, instrument, **dummy_inputs):
         ''' Returns current price (bid+ask) and home conversion factors.'''
+        
+        self.check_connection()
+        
         response = self.api.pricing.get(accountID = self.ACCOUNT_ID, 
                                    instruments = instrument
                                    )
@@ -52,6 +113,9 @@ class Oanda():
     
     def get_pending_orders(self, instrument=None):
         ''' Get all pending orders in the account. '''
+        
+        self.check_connection()
+        
         response = self.api.order.list_pending(accountID = self.ACCOUNT_ID, 
                                           instrument=instrument)
         
@@ -88,12 +152,17 @@ class Oanda():
     
     def cancel_pending_order(self, order_id):
         ''' Cancels pending order by ID. '''
+        
+        self.check_connection()
+        
         self.api.order.cancel(accountID = self.ACCOUNT_ID, 
                               orderSpecifier=str(order_id))
     
     def get_open_positions(self, instrument = None):
         ''' Gets the current positions open on the account. '''
         
+        self.check_connection()
+        
         response = self.api.trade.list_open(accountID=self.ACCOUNT_ID)
         
         oanda_open_trades = response.body['trades']
@@ -132,6 +201,8 @@ class Oanda():
         Parses order_details dict and handles order.
         '''
         
+        self.check_connection()
+        
         if order_details["order_type"] == 'market':
             response = self.place_market_order(order_details)
         elif order_details["order_type"] == 'stop-limit':
@@ -149,6 +220,9 @@ class Oanda():
         
     def place_market_order(self, order_details):
         ''' Places market order. '''
+        
+        self.check_connection()
+        
         stop_loss_details = self.get_stop_loss_details(order_details)
         take_profit_details = self.get_take_profit_details(order_details)
         
@@ -165,6 +239,9 @@ class Oanda():
         Places MarketIfTouchedOrder with Oanda.
         https://developer.oanda.com/rest-live-v20/order-df/
         '''
+        
+        self.check_connection()
+        
         stop_loss_details = self.get_stop_loss_details(order_details)
         take_profit_details = self.get_take_profit_details(order_details)
         
@@ -187,12 +264,16 @@ class Oanda():
     
     def place_limit_order(self, order_details):
         ''' (NOT YET IMPLEMENTED) PLaces limit order. '''
-        return
+        
+        self.check_connection()
+        
 
     def get_stop_loss_details(self, order_details):
         ''' Constructs stop loss details dictionary. '''
         # https://developer.oanda.com/rest-live-v20/order-df/#OrderType
         
+        self.check_connection()
+        
         if order_details["stop_type"] is not None:
             price = self.check_precision(order_details["instrument"], 
                                          order_details["stop_loss"])
@@ -210,6 +291,10 @@ class Oanda():
     
     def get_take_profit_details(self, order_details):
         ''' Constructs take profit details dictionary. '''
+        
+        self.check_connection()
+        
+        
         if order_details["take_profit"] is not None:
             price = self.check_precision(order_details["instrument"], 
                                          order_details["take_profit"])
@@ -221,6 +306,9 @@ class Oanda():
 
     def get_data(self, pair, period, interval):
         # print("Getting data for {}".format(pair))
+        
+        self.check_connection()
+        
         response    = self.api.instrument.candles(pair,
                                              granularity = interval,
                                              count = period,
@@ -234,6 +322,9 @@ class Oanda():
     
     def get_balance(self):
         ''' Returns account balance. '''
+        
+        self.check_connection()
+        
         response = self.api.account.get(accountID=self.ACCOUNT_ID)
         
         return response.body["account"].balance
@@ -241,6 +332,9 @@ class Oanda():
     
     def get_summary(self):
         ''' Returns account summary. '''
+        
+        self.check_connection()
+        
         # response = self.api.account.get(accountID=self.ACCOUNT_ID)
         response = self.api.account.summary(accountID=self.ACCOUNT_ID)
         # print(response.body['account'])
@@ -251,6 +345,9 @@ class Oanda():
     def close_position(self, instrument, long_units=None, short_units=None,
                        **dummy_inputs):
         ''' Closes all open positions on an instrument. '''
+        
+        self.check_connection()
+        
         # Check if the position is long or short
         
         # Temp code to close all positions
@@ -300,6 +397,9 @@ class Oanda():
     
     def get_precision(self, pair):
         ''' Returns the allowable precision for a given pair '''
+        
+        self.check_connection()
+        
         response = self.api.account.instruments(accountID = self.ACCOUNT_ID, 
                                                 instruments = pair)
         
@@ -333,6 +433,8 @@ class Oanda():
             api feed.
         '''
         
+        self.check_connection()
+        
         granularity_details = self.deconstruct_granularity(granularity)
         secs = granularity_details['seconds']
         mins = granularity_details['minutes']
@@ -383,6 +485,9 @@ class Oanda():
     
     
     def get_historical_data(self, pair, interval, from_time, to_time):
+        
+        self.check_connection()
+        
         response        = self.api.instrument.candles(pair,
                                                       granularity = interval,
                                                       fromTime = from_time,
@@ -490,7 +595,3 @@ class Oanda():
         
         return reduced_granularity
     
-    def get_NAV(self):
-        ''' Returns Net Asset Value of account. '''
-        response = self.api.account.get(accountID=self.ACCOUNT_ID)
-        return response.body["account"].NAV
