@@ -52,15 +52,17 @@ class ManageBot():
     
     """
     
-    def __init__(self, bot, home_dir, bot_name_string):
+    def __init__(self, bot, home_dir, bot_name_string, use_stream):
         
         self.bot = bot
         self.home_dir = home_dir
         self.managing = True
+        self.use_stream = use_stream
         
         self.active_bots_dir = os.path.join(home_dir, 'active_bots')
         self.active_bot_path = os.path.join(self.active_bots_dir, bot_name_string)
         self.killfile = os.path.join(self.home_dir, 'killbots')
+        self.suspendfile = os.path.join(self.home_dir, 'suspendbots')
         
         # Create name string
         self.bot_name_string = bot_name_string
@@ -93,6 +95,11 @@ class ManageBot():
         # Add bot to log
         self.write_bot_to_log()
         
+        # Signal that bot is ready to recieve data from stream
+        if self.use_stream:
+            self.bot._recieve_stream_data()
+        
+        # Manage
         while self.managing:
             
             # First check for any termination signals
@@ -119,51 +126,64 @@ class ManageBot():
                 
                 # End management
                 self.managing = False
+            
+            elif os.path.exists(self.suspendfile):
+                print("\nSuspending {} bot.".format(self.bot_name_string))
+                self.suspend()
+                print("Resuming {} bot.".format(self.bot_name_string))
                 
             else:
-                for atempt in range(10):
-                    try:
-                        # Refresh strategy with latest data
-                        self.bot._update_strategy_data()
-                        
-                        # Call bot update to act on latest data
-                        self.bot._update(-1)
-                    
-                    except BaseException as ex:
-                        # Get current system exception
-                        ex_type, ex_value, ex_traceback = sys.exc_info()
-                    
-                        # Extract unformatter stack traces as tuples
-                        trace_back = traceback.extract_tb(ex_traceback)
-                    
-                        # Format stacktrace
-                        stack_trace = list()
-                    
-                        for trace in trace_back:
-                            trade_string = "File : %s , Line : %d, " % (trace[0], trace[1]) + \
-                                           "Func.Name : %s, Message : %s" % (trace[2], trace[3])
-                            stack_trace.append(trade_string)
-                        
-                        print("WARNING FROM BOT MANAGER: The following exception was caught " +\
-                              "when updating {}.".format(self.bot_name_string))
-                        print("Exception type : %s " % ex_type.__name__)
-                        print("Exception message : %s" %ex_value)
-                        print("Stack trace : %s" %stack_trace)
-                        print("  Trying again.")
-                    
-                    else:
-                        break
-                    
-                else:
-                    print("FATAL: All attempts have failed. Going to sleep.")
+                # No termination signal detected, proceed to manage
                 
-                # Pause an amount, depending on granularity
-                base_granularity = self.bot.strategy_params['granularity'].split(',')[0]
-                if base_granularity == 'tick':
-                    time.sleep(1)
-                else:
-                    sleep_time = 0.25*self.granularity_to_seconds(base_granularity)
-                    time.sleep(sleep_time)
+                if not self.use_stream:
+                    # Periodic update mode
+                    for atempt in range(10):
+                        try:
+                            # Refresh strategy with latest data
+                            self.bot._update_strategy_data()
+                            
+                            # Call bot update to act on latest data
+                            self.bot._update(-1)
+                        
+                        except BaseException as ex:
+                            # Get current system exception
+                            ex_type, ex_value, ex_traceback = sys.exc_info()
+                        
+                            # Extract unformatter stack traces as tuples
+                            trace_back = traceback.extract_tb(ex_traceback)
+                        
+                            # Format stacktrace
+                            stack_trace = list()
+                        
+                            for trace in trace_back:
+                                trade_string = "File : %s , Line : %d, " % (trace[0], trace[1]) + \
+                                               "Func.Name : %s, Message : %s" % (trace[2], trace[3])
+                                stack_trace.append(trade_string)
+                            
+                            print("WARNING FROM BOT MANAGER: The following exception was caught " +\
+                                  "when updating {}.".format(self.bot_name_string))
+                            print("Exception type : %s " % ex_type.__name__)
+                            print("Exception message : %s" %ex_value)
+                            print("Stack trace : %s" %stack_trace)
+                            print("  Trying again.")
+                        
+                        else:
+                            break
+                        
+                    else:
+                        print("FATAL: All attempts have failed. Going to sleep.")
+                    
+                    # Pause an amount, depending on granularity
+                    base_granularity = self.bot.strategy_params['granularity'].split(',')[0]
+                    if base_granularity == 'tick':
+                        time.sleep(1)
+                    else:
+                        sleep_time = 0.25*self.granularity_to_seconds(base_granularity)
+                        time.sleep(sleep_time)
+
+    def suspend(self):
+        while os.path.exists(self.suspendfile):
+            pass    
             
     def write_bot_to_log(self):
         '''
