@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-Custom indicators
-
+Custom indicators library.
 """
-import talib as ta
-import math
+
+from finta import TA
 import numpy as np
 import pandas as pd
 
@@ -12,8 +11,7 @@ import pandas as pd
 def supertrend(data, period = 10, ATR_multiplier = 3.0):
     ''' Based on the SuperTrend indicator by KivancOzbilgic on TradingView '''
     hl2             = (data.High.values + data.Low.values) / 2
-    true_range      = ta.TRANGE(data.High.values, data.Low.values, data.Close.values)
-    atr             = ta.SMA(true_range, timeperiod=period)
+    atr             = TA.ATR(data, period)
     source          = hl2
     
     up              = source - (ATR_multiplier*atr)
@@ -80,7 +78,7 @@ def supertrend(data, period = 10, ATR_multiplier = 3.0):
 
 def stoch_rsi(data, K_period=3, D_period=3, RSI_length=14, Stochastic_length=14):
 
-    rsi1 = ta.RSI(data, timeperiod = RSI_length)
+    rsi1 = TA.RSI(data, period=RSI_length)
     stoch = stochastic(rsi1, rsi1, rsi1, Stochastic_length)
     
     K = sma(stoch, K_period)
@@ -112,16 +110,42 @@ def sma(data, period=14):
     
     return sma_list
 
-# def ema(data, period):
-    
-#     sma_list = []
-    
-#     for i in range(len(data)):
-#         average = sum(data[i-period+1:i+1])/period
-#         sma_list.append(average)
-    
-#     return sma_list
 
+def ema(data, period=14, smoothing=2):
+    
+    ema = [sum(data[:period]) / period]
+    
+    for price in data[period:]:
+        ema.append((price * (smoothing / (1 + period))) + ema[-1] * (1 - (smoothing / (1 + period))))
+    
+    for i in range(period-1):
+        ema.insert(0, np.nan)
+    
+    return ema
+
+
+def true_range(data, period=14):
+    high_low = data['High'] - data['Low']
+    high_close = np.abs(data['High'] - data['Close'].shift())
+    low_close = np.abs(data['Low'] - data['Close'].shift())
+    
+    ranges = pd.concat([high_low, high_close, low_close], axis=1)
+    true_range = np.max(ranges, axis=1)
+    
+    return true_range
+
+
+def atr(data, period=14):
+    
+    tr = true_range(data, period)
+    
+    atr = tr.rolling(period).sum()/period
+    
+    return atr
+
+
+def rsi():
+    return
 
 def crossover(list_1, list_2):
     ''' 
@@ -235,7 +259,7 @@ def bullish_engulfing(data, detection = None):
     body_low        = np.minimum(data.Close.values, data.Open.values)
     body            = body_high - body_low
     
-    body_avg        = ta.EMA(body, body_len)
+    body_avg        = ema(body, body_len)
     short_body      = body < body_avg
     long_body       = body > body_avg
     up_shadow       = data.High.values - body_high
@@ -310,7 +334,7 @@ def bearish_engulfing(data, detection = None):
     body_low        = np.minimum(data.Close.values, data.Open.values)
     body            = body_high - body_low
     
-    body_avg        = ta.EMA(body, body_len)
+    body_avg        = ema(body, body_len)
     short_body      = body < body_avg
     long_body       = body > body_avg
     up_shadow       = data.High.values - body_high
@@ -378,17 +402,19 @@ def find_swings(data, use_body = False):
     '''
         Locates the recent swings in price and returns a rolling list of 
         swing prices.
-        
     '''
+    
+    # TODO - include resampling option to find swings on higher timeframe
+    
     # use_body flag currently not implemented. Idea is to use body close/open
     # instead of high/low (wicks) of candle for swings.
     
     # oc2     = (data.Open.values + data.Close.values)/2
     hl2     = (data.High.values + data.Low.values)/2
     n       = 2
-    ema     = ta.EMA(hl2, n)
+    EMA     = ema(hl2, n)
     
-    grad    = np.gradient(ema)
+    grad    = np.gradient(EMA)
     
     swings   = np.zeros(len(grad))
     for i in range(1, len(grad)):
@@ -624,7 +650,7 @@ def build_multiplier_grid(origin, direction, multiplier, no_levels, precision, s
     levels = [i for i in range(1, no_levels + 1)]
 
     pos_levels = [round(origin + direction*spacing*i, precision) for i in levels]
-    neg_spaces = [spacing*multiplier**(i-1) for i in levels]
+    neg_spaces = [spacing*multiplier**(i) for i in levels]
     neg_levels = []
     prev_neg_level = origin
     for i in range(len(levels)):
